#include-once
#include <A3LString.au3>
#include <A3LWinAPI.au3>

Opt("MustDeclareVars", 1)

; #INDEX# =======================================================================================================================
; Title .........: GDI+
; Description ...: Microsoft Windows GDI+ is a class-based API for C/C++ programmers. It enables applications to use graphics and
;                  formatted text on both the video display and the printer. Applications based on the Microsoft Win32 API do not
;                  access graphics hardware directly. Instead, GDI+ interacts with device drivers on behalf of applications. GDI+
;                  can be used in all Windows-based applications.  GDI+ is new technology that is included in Windows XP and  the
;                  Windows Server 2003.  It is required as a redistributable for applications that run on the Windows NT 4.0 SP6,
;                  Windows 2000, Windows 98, and Windows Me operating systems.
; Author ........: Paul Campbell (PaulIA)
; ===============================================================================================================================

; #CONSTANTS# ===================================================================================================================
  ; Pen Dash Cap Types
  Global Const $GDIP_DASHCAPFLAT                = 0             ; A square cap that squares off both ends of each dash
  Global Const $GDIP_DASHCAPROUND               = 2             ; A circular cap that rounds off both ends of each dash
  Global Const $GDIP_DASHCAPTRIANGLE            = 3             ; A triangular cap that points both ends of each dash

  ; Pen Dash Style Types
  Global Const $GDIP_DASHSTYLESOLID             = 0             ; A solid line
  Global Const $GDIP_DASHSTYLEDASH              = 1             ; A dashed line
  Global Const $GDIP_DASHSTYLEDOT               = 2             ; A dotted line
  Global Const $GDIP_DASHSTYLEDASHDOT           = 3             ; An alternating dash-dot line
  Global Const $GDIP_DASHSTYLEDASHDOTDOT        = 4             ; An alternating dash-dot-dot line
  Global Const $GDIP_DASHSTYLECUSTOM            = 5             ; A a user-defined, custom dashed line

  ; Enocder Parameter GUIDs
  Global Const $GDIP_EPGCHROMINANCETABLE        = '{F2E455DC-09B3-4316-8260-676ADA32481C}'
  Global Const $GDIP_EPGCOLORDEPTH              = '{66087055-AD66-4C7C-9A18-38A2310B8337}'
  Global Const $GDIP_EPGCOMPRESSION             = '{E09D739D-CCD4-44EE-8EBA-3FBF8BE4FC58}'
  Global Const $GDIP_EPGLUMINANCETABLE          = '{EDB33BCE-0266-4A77-B904-27216099E717}'
  Global Const $GDIP_EPGQUALITY                 = '{1D5BE4B5-FA4A-452D-9CDD-5DB35105E7EB}'
  Global Const $GDIP_EPGRENDERMETHOD            = '{6D42C53A-229A-4825-8BB7-5C99E2B9A8B8}'
  Global Const $GDIP_EPGSAVEFLAG                = '{292266FC-AC40-47BF-8CFC-A85B89A655DE}'
  Global Const $GDIP_EPGSCANMETHOD              = '{3A4E2661-3109-4E56-8536-42C156E7DCFA}'
  Global Const $GDIP_EPGTRANSFORMATION          = '{8D0EB2D1-A58E-4EA8-AA14-108074B7B6F9}'
  Global Const $GDIP_EPGVERSION                 = '{24D18C76-814A-41A4-BF53-1C219CCCF797}'

  ; Encoder Parameter Types
  Global Const $GDIP_EPTBYTE                    = 1             ; 8 bit unsigned integer
  Global Const $GDIP_EPTASCII                   = 2             ; Null terminated character string
  Global Const $GDIP_EPTSHORT                   = 3             ; 16 bit unsigned integer
  Global Const $GDIP_EPTLONG                    = 4             ; 32 bit unsigned integer
  Global Const $GDIP_EPTRATIONAL                = 5             ; Two longs (numerator, denomintor)
  Global Const $GDIP_EPTLONGRANGE               = 6             ; Two longs (low, high)
  Global Const $GDIP_EPTUNDEFINED               = 7             ; Array of bytes of any type
  Global Const $GDIP_EPTRATIONALRANGE           = 8             ; Two ratationals (low, high)

  ; GDI Error Codes
  Global Const $GDIP_ERROK                      = 0             ; Method call was successful
  Global Const $GDIP_ERRGENERICERROR            = 1             ; Generic method call error
  Global Const $GDIP_ERRINVALIDPARAMETER        = 2             ; One of the arguments passed to the method was not valid
  Global Const $GDIP_ERROUTOFMEMORY             = 3             ; The operating system is out of memory
  Global Const $GDIP_ERROBJECTBUSY              = 4             ; One of the arguments in the call is already in use
  Global Const $GDIP_ERRINSUFFICIENTBUFFER      = 5             ; A buffer is not large enough
  Global Const $GDIP_ERRNOTIMPLEMENTED          = 6             ; The method is not implemented
  Global Const $GDIP_ERRWIN32ERROR              = 7             ; The method generated a Microsoft Win32 error
  Global Const $GDIP_ERRWRONGSTATE              = 8             ; The object is in an invalid state to satisfy the API call
  Global Const $GDIP_ERRABORTED                 = 9             ; The method was aborted
  Global Const $GDIP_ERRFILENOTFOUND            = 10            ; The specified image file or metafile cannot be found
  Global Const $GDIP_ERRVALUEOVERFLOW           = 11            ; The method produced a numeric overflow
  Global Const $GDIP_ERRACCESSDENIED            = 12            ; A write operation is not allowed on the specified file
  Global Const $GDIP_ERRUNKNOWNIMAGEFORMAT      = 13            ; The specified image file format is not known
  Global Const $GDIP_ERRFONTFAMILYNOTFOUND      = 14            ; The specified font family cannot be found
  Global Const $GDIP_ERRFONTSTYLENOTFOUND       = 15            ; The specified style is not available for the specified font
  Global Const $GDIP_ERRNOTTRUETYPEFONT         = 16            ; The font retrieved is not a TrueType font
  Global Const $GDIP_ERRUNSUPPORTEDGDIVERSION   = 17            ; The installed GDI+ version is incompatible
  Global Const $GDIP_ERRGDIPLUSNOTINITIALIZED   = 18            ; The GDI+ API is not in an initialized state
  Global Const $GDIP_ERRPROPERTYNOTFOUND        = 19            ; The specified property does not exist in the image
  Global Const $GDIP_ERRPROPERTYNOTSUPPORTED    = 20            ; The specified property is not supported

  ; Encoder Value Types
  Global Const $GDIP_EVTCOMPRESSIONLZW          = 2             ; TIFF: LZW compression
  Global Const $GDIP_EVTCOMPRESSIONCCITT3       = 3             ; TIFF: CCITT3 compression
  Global Const $GDIP_EVTCOMPRESSIONCCITT4       = 4             ; TIFF: CCITT4 compression
  Global Const $GDIP_EVTCOMPRESSIONRLE          = 5             ; TIFF: RLE compression
  Global Const $GDIP_EVTCOMPRESSIONNONE         = 6             ; TIFF: No compression
  Global Const $GDIP_EVTTRANSFORMROTATE90       = 13            ; JPEG: Lossless 90 degree clockwise rotation
  Global Const $GDIP_EVTTRANSFORMROTATE180      = 14            ; JPEG: Lossless 180 degree clockwise rotation
  Global Const $GDIP_EVTTRANSFORMROTATE270      = 15            ; JPEG: Lossless 270 degree clockwise rotation
  Global Const $GDIP_EVTTRANSFORMFLIPHORIZONTAL = 16            ; JPEG: Lossless horizontal flip
  Global Const $GDIP_EVTTRANSFORMFLIPVERTICAL   = 17            ; JPEG: Lossless vertical flip
  Global Const $GDIP_EVTMULTIFRAME              = 18            ; Multiple frame encoding
  Global Const $GDIP_EVTLASTFRAME               = 19            ; Last frame of a multiple frame image
  Global Const $GDIP_EVTFLUSH                   = 20            ; The encoder object is to be closed
  Global Const $GDIP_EVTFRAMEDIMENSIONPAGE      = 23            ; TIFF: Page frame dimension

  ; Image Codec Flags constants
  Global Const $GDIP_ICFENCODER                 = 0x00000001    ; The codec supports encoding (saving)
  Global Const $GDIP_ICFDECODER                 = 0x00000002    ; The codec supports decoding (reading)
  Global Const $GDIP_ICFSUPPORTBITMAP           = 0x00000004    ; The codec supports raster images (bitmaps)
  Global Const $GDIP_ICFSUPPORTVECTOR           = 0x00000008    ; The codec supports vector images (metafiles)
  Global Const $GDIP_ICFSEEKABLEENCODE          = 0x00000010    ; The encoder requires a seekable output stream
  Global Const $GDIP_ICFBLOCKINGDECODE          = 0x00000020    ; The decoder has blocking behavior during the decoding process
  Global Const $GDIP_ICFBUILTIN                 = 0x00010000    ; The codec is built in to GDI+
  Global Const $GDIP_ICFSYSTEM                  = 0x00020000    ; Not used in GDI+ version 1.0
  Global Const $GDIP_ICFUSER                    = 0x00040000    ; Not used in GDI+ version 1.0

  ; Image Lock Mode constants
  Global Const $GDIP_ILMREAD                    = 0x0001        ; A portion of the image is locked for reading
  Global Const $GDIP_ILMWRITE                   = 0x0002        ; A portion of the image is locked for writing
  Global Const $GDIP_ILMUSERINPUTBUF            = 0x0004        ; The buffer is allocated by the user

  ; LineCap constants
  Global Const $GDIP_LINECAPFLAT                = 0x00          ; Specifies a flat cap
  Global Const $GDIP_LINECAPSQUARE              = 0x01          ; Specifies a square cap
  Global Const $GDIP_LINECAPROUND               = 0x02          ; Specifies a circular cap
  Global Const $GDIP_LINECAPTRIANGLE            = 0x03          ; Specifies a triangular cap
  Global Const $GDIP_LINECAPNOANCHOR            = 0x10          ; Specifies that the line ends are not anchored
  Global Const $GDIP_LINECAPSQUAREANCHOR        = 0x11          ; Specifies that the line ends are anchored with a square
  Global Const $GDIP_LINECAPROUNDANCHOR         = 0x12          ; Specifies that the line ends are anchored with a circle
  Global Const $GDIP_LINECAPDIAMONDANCHOR       = 0x13          ; Specifies that the line ends are anchored with a diamond
  Global Const $GDIP_LINECAPARROWANCHOR         = 0x14          ; Specifies that the line ends are anchored with arrowheads
  Global Const $GDIP_LINECAPCUSTOM              = 0xFF          ; Specifies that the line ends are made from a CustomLineCap

  ; Pixel Format constants
  GLOBAL CONST $GDIP_PXF01INDEXED               = 0x00030101    ; 1 bpp, indexed
  GLOBAL CONST $GDIP_PXF04INDEXED               = 0x00030402    ; 4 bpp, indexed
  GLOBAL CONST $GDIP_PXF08INDEXED               = 0x00030803    ; 8 bpp, indexed
  GLOBAL CONST $GDIP_PXF16GRAYSCALE             = 0x00101004    ; 16 bpp, grayscale
  GLOBAL CONST $GDIP_PXF16RGB555                = 0x00021005    ; 16 bpp; 5 bits for each RGB
  GLOBAL CONST $GDIP_PXF16RGB565                = 0x00021006    ; 16 bpp; 5 bits red, 6 bits green, and 5 bits blue
  GLOBAL CONST $GDIP_PXF16ARGB1555              = 0x00061007    ; 16 bpp; 1 bit for alpha and 5 bits for each RGB component
  GLOBAL CONST $GDIP_PXF24RGB                   = 0x00021808    ; 24 bpp; 8 bits for each RGB
  GLOBAL CONST $GDIP_PXF32RGB                   = 0x00022009    ; 32 bpp; 8 bits for each RGB. No alpha.
  GLOBAL CONST $GDIP_PXF32ARGB                  = 0x0026200A    ; 32 bpp; 8 bits for each RGB and alpha
  GLOBAL CONST $GDIP_PXF32PARGB                 = 0x000D200B    ; 32 bpp; 8 bits for each RGB and alpha, pre-mulitiplied
  GLOBAL CONST $GDIP_PXF48RGB                   = 0x0010300C    ; 48 bpp; 16 bits for each RGB
  GLOBAL CONST $GDIP_PXF64ARGB                  = 0x0034400D    ; 64 bpp; 16 bits for each RGB and alpha
  GLOBAL CONST $GDIP_PXF64PARGB                 = 0x001C400E    ; 64 bpp; 16 bits for each RGB and alpha, pre-multiplied
; ===============================================================================================================================

; #STRUCTURE# ===================================================================================================================
; Description ...: tagGDIPBITMAPDATA structure
; Fields ........: Width    - Number of pixels in one scan line of the bitmap
;                  Height   - Number of scan lines in the bitmap
;                  Stride   - Offset, in bytes, between consecutive scan lines of the bitmap.  If the  stride  is  positive,  the
;                  +bitmap is top-down. If the stride is negative, the bitmap is bottom-up
;                  Format   - Specifies the pixel format of the bitmap
;                  Scan0    - Pointer to the first (index 0) scan line of the bitmap
;                  Reserved - Reserved for future use
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; ===============================================================================================================================
Global Const $tagGDIPBITMAPDATA = "uint Width;uint Height;int Stride;uint Format;ptr Scan0;ptr Reserved"

; #STRUCTURE# ===================================================================================================================
; Description ...: tagGDIPENCODERPARAM structure
; Fields ........: GUID   - Indentifies the parameter category (GDI_EPG constants)
;                  Count  - Number of values in the array pointed to by the Value member
;                  Type   - Identifies the data type of the parameters (GDI_EPT constants)
;                  Values - Pointer to an array of values. Each value has the type specified by the Type member.
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; ===============================================================================================================================
Global Const $tagGDIPENCODERPARAM = "byte GUID[16];dword Count;dword Type;ptr Values"

; #STRUCTURE# ===================================================================================================================
; Description ...: tagGDIPPENCODERPARAMS structure
; Fields ........: Count  - Number of tagGDIPENCODERPARAM structures in the array
;                  Params - Start of tagGDIPENCODERPARAM structures
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; ===============================================================================================================================
Global Const $tagGDIPPENCODERPARAMS = "dword Count;byte Params[0]"

; #STRUCTURE# ===================================================================================================================
; Description ...: tagGDIPIMAGECODECINFO structure
; Fields ........: CLSID      - Codec identifier (GUID structure)
;                  FormatID   - File format identifier (GUID structure)
;                  CodecName  - Pointer to a Unicode null terminated string that contains the codec name
;                  DllName    - Pointer to a Unicode null terminated string that contains the path name of the DLL in  which  the
;                  +codec resides. If the codec is not in a DLL, this pointer is 0.
;                  FormatDesc - Pointer to a Unicode null terminated string that contains the name of the file format used by the
;                  +codec.
;                  FileExt    - Pointer to a Unicode null terminated string that contains all filename extensions associated with
;                  +the codec. The extensions are separated by semicolons.
;                  MimeType   - Pointer to a null-terminated string that contains the mime type of the codec
;                  Flags      - Combination of $GDIP_ICF flags
;                  Version    - Indicates the version of the codec
;                  SigCount   - Indicates the number of signatures used by the file format associated with the codec
;                  SigSize    - Indicates the number of bytes in each signature
;                  SigPattern - Pointer to an array of bytes that contains the pattern for each signature
;                  SigMask    - Pointer to an array of bytes that contains the mask for each signature
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; ===============================================================================================================================
Global Const $tagGDIPIMAGECODECINFO = "byte CLSID[16];byte FormatID[16];ptr CodecName;ptr DllName;ptr FormatDesc;ptr FileExt;" & _
             "ptr MimeType;dword Flags;dword Version;dword SigCount;dword SigSize;ptr SigPattern;ptr SigMask"

; #STRUCTURE# ===================================================================================================================
; Description ...: tagGDIPRECTF structure
; Fields ........: X      - X coordinate of upper left hand corner of rectangle
;                  Y      - Y coordinate of upper left hand corner of rectangle
;                  Width  - Rectangle width
;                  Height - Rectangle height
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; ===============================================================================================================================
Global Const $tagGDIPRECTF = "float X;float Y;float Width;float Height"

; #STRUCTURE# ===================================================================================================================
; Description ...: tagGDIPSTARTUPINPUT structure
; Fields ........: Version  - Specifies the version of Microsoft Windows GDI+
;                  Callback - Pointer to a callback function that GDI+ can call, on debug builds, for assertions and warnings
;                  NoThread - Boolean value that specifies whether to suppress the GDI+ background thread. If you set this member
;                  +to True, GdiplusStartup returns a pointer to a hook function and a pointer to an  unhook  function.  You must
;                  +call those functions appropriately to replace the background thread. If you do not want to be responsible for
;                  +calling the hook and unhook functions, set this member to False.
;                  NoCodecs - Boolean value that specifies whether you want GDI+ to suppress external image codecs.  GDI+ version
;                  +1.0 does not support external image codecs, so this parameter is ignored.
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; ===============================================================================================================================
Global Const $tagGDIPSTARTUPINPUT = "int Version;int Callback;int NoThread;int NoCodecs"

; #STRUCTURE# ===================================================================================================================
; Description ...: tagGDIPSTARTUPOUTPUT structure
; Fields ........: HookProc   - Receives a pointer to a hook function
;                  UnhookProc - Receives a pointer to an unhook function
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; ===============================================================================================================================
Global Const $tagGDIPSTARTUPOUTPUT = "int HookProc;int UnhookProc"

; #VARIABLES# ===================================================================================================================
  Global $ghA3LGDIPBrush = 0
  Global $ghA3LGDIPDll   = 0
  Global $ghA3LGDIPPen   = 0
  Global $giA3LGDIPRef   = 0
  Global $giA3LGDIPToken = 0
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description ...: Creates an adjustable arrow line cap with the specified height and width
; Parameters ....: $nHeight - Specifies the length, in units, of the arrow from its base to its point
;                  $nWidth  - Specifies the distance, in units, between the corners of the base of the arrow
;                  $fFilled - Fill flag:
;                  | True - Arrow will be filled
;                  |False - Arrow will not be filled
; Return values .: Success      - Returns a handle to a new ArrowCap object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: After you are done with the object, call _GDIP_ArrowCapDispose to release the object resources
; Related .......: _GDIP_ArrowCapDispose
; ===============================================================================================================================
Func _GDIP_ArrowCapCreate($nHeight, $nWidth, $fFilled=True)
  Local $iHeight, $iWidth, $aResult

  $iHeight = _Lib_FloatToInt($nHeight)
  $iWidth  = _Lib_FloatToInt($nWidth )
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateAdjustableArrowCap", "int", $iHeight, "int", $iWidth, "int", $fFilled, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[4])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release an adjustable arrow line cap object
; Parameters ....: $hCap        - Handle to a adjustable arrow line cap object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ArrowCapCreate
; ===============================================================================================================================
Func _GDIP_ArrowCapDispose($hCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteCustomLineCap", "hwnd", $hCap)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Determines whether the arrow cap is filled
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
; Return values .: True         - Arrow cap is filled
;                  False        - Arrow cap is not filled
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ArrowCapSetFillState
; ===============================================================================================================================
Func _GDIP_ArrowCapGetFillState($hArrowCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetAdjustableArrowCapFillState", "hwnd", $hArrowCap, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2]<>0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the height of the arrow cap
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
; Return values .: Success      - Returns the height of the arrow cap
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: The height is the distance from the base of the arrow to its vertex
; Related .......: _GDIP_ArrowCapSetHeight
; ===============================================================================================================================
Func _GDIP_ArrowCapGetHeight($hArrowCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetAdjustableArrowCapHeight", "hwnd", $hArrowCap, "int_ptr", 0)
  Return SetError($aResult[0], 0, _Lib_IntToFloat($aResult[2]))
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the value of the inset
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
; Return values .: Success      - Inset value
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: The middle inset is the number of units that the midpoint of the base shifts  towards  the  vertex.  A  middle
;                  inset of zero results in no shift (the base is a straight line,  giving  the  arrow  a  triangular  shape).  A
;                  positive (greater than zero) middle inset results in a shift the specified number of units toward  the  vertex
;                  (the base is an arrow shape that points toward the vertex, giving the arrow cap a V-shape).  A negative  (less
;                  than zero) middle inset results in a shift the specified number  of  units  away  from  the  vertex (the  base
;                  becomes an arrow shape that points away from the vertex, giving the arrow  either  a  diamond  shape  (if  the
;                  absolute value of the middle inset is equal to the height) or distorted diamond shape.  If the middle inset is
;                  equal to or greater than the height of the arrow cap, the cap does not appear at all.  The value of the middle
;                  inset affects the arrow cap only if the arrow cap is filled.
; Related .......: _GDIP_ArrowCapSetMiddleInset
; ===============================================================================================================================
Func _GDIP_ArrowCapGetMiddleInset($hArrowCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetAdjustableArrowCapMiddleInset", "hwnd", $hArrowCap, "int_ptr", 0)
  Return SetError($aResult[0], 0, _Lib_IntToFloat($aResult[2]))
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the width of the arrow cap
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
; Return values .: Success      - Returns the width of the arrow cap
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: The width is the distance between the endpoints of the base of the arrow
; Related .......: _GDIP_ArrowCapSetWidth
; ===============================================================================================================================
Func _GDIP_ArrowCapGetWidth($hArrowCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetAdjustableArrowCapWidth", "hwnd", $hArrowCap, "int_ptr", 0)
  Return SetError($aResult[0], 0, _Lib_IntToFloat($aResult[2]))
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets whether the arrow cap is filled
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
;                  $fFilled     - Fill state:
;                  | True - Arrow cap is filled
;                  |False - Arrow cap is not filled
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ArrowCapGetFillState
; ===============================================================================================================================
Func _GDIP_ArrowCapSetFillState($hArrowCap, $fFilled=True)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetAdjustableArrowCapFillState", "hwnd", $hArrowCap, "int", $fFilled)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the height of the arrow cap
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
;                  $nHeight     - Specifies the length, in units, of the arrow from its base to its point
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ArrowCapGetHeight
; ===============================================================================================================================
Func _GDIP_ArrowCapSetHeight($hArrowCap, $nHeight)
  Local $iHeight, $aResult

  $iHeight = _Lib_FloatToInt($nHeight)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetAdjustableArrowCapHeight", "hwnd", $hArrowCap, "int", $iHeight)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the value of the inset
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
;                  $nInset      - Inset value
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: The middle inset is the number of units that the midpoint of the base shifts  towards  the  vertex.  A  middle
;                  inset of zero results in no shift (the base is a straight line,  giving  the  arrow  a  triangular  shape).  A
;                  positive (greater than zero) middle inset results in a shift the specified number of units toward  the  vertex
;                  (the base is an arrow shape that points toward the vertex, giving the arrow cap a V-shape).  A negative  (less
;                  than zero) middle inset results in a shift the specified number  of  units  away  from  the  vertex (the  base
;                  becomes an arrow shape that points away from the vertex, giving the arrow  either  a  diamond  shape  (if  the
;                  absolute value of the middle inset is equal to the height) or distorted diamond shape.  If the middle inset is
;                  equal to or greater than the height of the arrow cap, the cap does not appear at all.  The value of the middle
;                  inset affects the arrow cap only if the arrow cap is filled.
; Related .......: _GDIP_ArrowCapGetMiddleInset
; ===============================================================================================================================
Func _GDIP_ArrowCapSetMiddleInset($hArrowCap, $nInset)
  Local $iInset, $aResult

  $iInset  = _Lib_FloatToInt($nInset)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetAdjustableArrowCapMiddleInset", "hwnd", $hArrowCap, "int", $iInset)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the width of the arrow cap
; Parameters ....: $hArrowCap   - Handle to a ArrowCap object
;                  $nWidth      - Specifies the width, in units, of the arrow between the endpoints of the base of the arrow
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ArrowCapGetWidth
; ===============================================================================================================================
Func _GDIP_ArrowCapSetWidth($hArrowCap, $nWidth)
  Local $iWidth, $aResult

  $iWidth  = _Lib_FloatToInt($nWidth)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetAdjustableArrowCapWidth", "hwnd", $hArrowCap, "int", $iWidth)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a clone of a Bitmap object from the coordinates and format specified
; Parameters ....: $hBmp        - Handle to a Bitmap object
;                  $iLeft       - X coordinate of upper left corner of the rectangle to copy
;                  $iTop        - Y coordinate of upper left corner of the rectangle to copy
;                  $iWidth      - The width of the rectangle that specifies the portion of this bitmap to copy
;                  $iHeight     - The height of the rectangle that specifies the portion of this bitmap to copy
;                  $iFormat     - Pixel format for the new bitmap:
;                  |$GDIP_PXF01INDEXED   = 1 bit per pixel, indexed
;                  |$GDIP_PXF04INDEXED   = 4 bits per pixel, indexed
;                  |$GDIP_PXF08INDEXED   = 8 bits per pixel, indexed
;                  |$GDIP_PXF16GRAYSCALE = 16 bits per pixel, grayscale
;                  |$GDIP_PXF16RGB555    = 16 bits per pixel; 5 bits for each RGB component
;                  |$GDIP_PXF16RGB565    = 16 bits per pixel; 5 bits for red, 6 bits for green and 5 bits blue
;                  |$GDIP_PXF16ARGB1555  = 16 bits per pixel; 1 bit for alpha and 5 bits for each RGB component
;                  |$GDIP_PXF24RGB       = 24 bits per pixel; 8 bits for each RGB component
;                  |$GDIP_PXF32RGB       = 32 bits per pixel; 8 bits for each RGB component. No alpha component.
;                  |$GDIP_PXF32ARGB      = 32 bits per pixel; 8 bits for each RGB and alpha component
;                  |$GDIP_PXF32PARGB     = 32 bits per pixel; 8 bits for each RGB and alpha component, pre-mulitiplied
;                  |$GDIP_PXF48RGB       = 48 bits per pixel; 16 bits for each RGB component
;                  |$GDIP_PXF64ARGB      = 64 bits per pixel; 16 bits for each RGB and alpha component
;                  |$GDIP_PXF64PARGB     = 64 bits per pixel; 16 bits for each RGB and alpha component, pre-multiplied
; Return values .: Success      - Returns a handle to a new Bitmap object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Bitmap object, call _API_DeleteObject to release the resources
; Related .......: _API_DeleteObject
; ===============================================================================================================================
Func _GDIP_BitmapCloneArea($hBmp, $iLeft, $iTop, $iWidth, $iHeight, $iFormat=0x00021808)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCloneBitmapAreaI", "int", $iLeft, "int", $iTop, "int", $iWidth, "int", $iHeight, _
                    "int", $iFormat, "ptr", $hBmp, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[7])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a Bitmap object from file
; Parameters ....: $sFileName   - Path to a bitmap file
; Return values .: Success      - Handle to a Bitmap object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Bitmap object, call _API_DeleteObject to release the resources
; Related .......: _API_DeleteObject
; ===============================================================================================================================
Func _GDIP_BitmapCreateFromFile($sFileName)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateBitmapFromFile", "wstr", $sFileName, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Creates a Bitmap object based on a Graphics object, a width, and a height
; Parameters ....: $iWidth      - Specifies the width, in pixels, of the bitmap
;                  $iHeight     - Specifies the height, in pixels, of the bitmap
;                  $hGraphics   - Handle to a Graphics object
; Return values .: Success      - Handle to a Bitmap object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Bitmap object, call _API_DeleteObject to release the resources
; Related .......: _API_DeleteObject
; ===============================================================================================================================
Func _GDIP_BitmapCreateFromGraphics($iWidth, $iHeight, $hGraphics)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateBitmapFromGraphics", "int", $iWidth, "int", $iHeight, "hwnd", $hGraphics, _
                     "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[4])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a Bitmap object from a bitmap handle
; Parameters ....: $hBmp        - Handle to a HBITMAP
;                  $hPal        - Handle to a HPALETTE
; Return values .: Success      - Handle to a Bitmap object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Bitmap object, call _API_DeleteObject to release the resources
; Related .......: _API_DeleteObject
; ===============================================================================================================================
Func _GDIP_BitmapCreateFromHBITMAP($hBmp, $hPal=0)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateBitmapFromHBITMAP", "hwnd", $hBmp, "hwnd", $hPal, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[3])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a handle to a bitmap from a bitmap object
; Parameters ....: $hBitmap     - Handle to a bitmap object
;                  $iARGB       - Color object that specifies the background color
; Return values .: Success      - Handle to a HBITMAP
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Bitmap object, call _API_DeleteObject to release the resources
; Related .......: _API_DeleteObject
; ===============================================================================================================================
Func _GDIP_BitmapCreateHBITMAPFromBitmap($hBitmap, $iARGB=0xFF000000)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateHBITMAPFromBitmap", "hwnd", $hBitmap, "int_ptr", 0, "int", $iARGB)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a bitmap object
; Parameters ....: $hBitmap     - Handle to a bitmap object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_BitmapDispose($hBitmap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDisposeImage", "hwnd", $hBitmap)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Locks a portion of a bitmap for reading or writing
; Parameters ....: $hBitmap     - Handle to a bitmap object
;                  $iLeft       - X coordinate of the upper-left corner of the rectangle to lock
;                  $iTop        - Y coordinate of the upper-left corner of the rectangle to lock
;                  $iRight      - X coordinate of the lower-right corner of the rectangle to lock
;                  $iBottom     - Y coordinate of the lower-right corner of the rectangle to lock
;                  $iFlags      - Set of flags that specify whether the locked portion of the bitmap is available for reading  or
;                  +for writing and whether the caller has already allocated a buffer. Can be a combination of the following:
;                  |$GDIP_ILMREAD         - A portion of the image is locked for reading
;                  |$GDIP_ILMWRITE        - A portion of the image is locked for writing
;                  |$GDIP_ILMUSERINPUTBUF - The buffer is allocated by the user
;                  $iFormat     - Specifies the format of the pixel data in the temporary buffer. Can be one of the following:
;                  |$GDIP_PXF01INDEXED   - 1 bpp, indexed
;                  |$GDIP_PXF04INDEXED   - 4 bpp, indexed
;                  |$GDIP_PXF08INDEXED   - 8 bpp, indexed
;                  |$GDIP_PXF16GRAYSCALE - 16 bpp, grayscale
;                  |$GDIP_PXF16RGB555    - 16 bpp; 5 bits for each RGB
;                  |$GDIP_PXF16RGB565    - 16 bpp; 5 bits red, 6 bits green, and 5 bits blue
;                  |$GDIP_PXF16ARGB1555  - 16 bpp; 1 bit for alpha and 5 bits for each RGB component
;                  |$GDIP_PXF24RGB       - 24 bpp; 8 bits for each RGB
;                  |$GDIP_PXF32RGB       - 32 bpp; 8 bits for each RGB. No alpha.
;                  |$GDIP_PXF32ARGB      - 32 bpp; 8 bits for each RGB and alpha
;                  |$GDIP_PXF32PARGB     - 32 bpp; 8 bits for each RGB and alpha, pre-mulitiplied
; Return values .: Success      - tagGDIPBITMAPDATA structure
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the locked portion, call _GDIP_BitmapUnlockBits to release the locked region
; Related .......: _API_DeleteObject
; ===============================================================================================================================
Func _GDIP_BitmapLockBits($hBitmap, $iLeft, $iTop, $iRight, $iBottom, $iFlags=1, $iFormat=0x00022009)
  Local $pData, $tData, $pRect, $tRect, $aResult

  $tData = DllStructCreate($tagGDIPBITMAPDATA)
  $pData = DllStructGetPtr($tData)
  $tRect = DllStructCreate($tagRECT)
  $pRect = DllStructGetPtr($tRect)
  DllStructSetData($tRect, "Left"  , $iLeft  )
  DllStructSetData($tRect, "Top"   , $iTop   )
  DllStructSetData($tRect, "Right" , $iRight )
  DllStructSetData($tRect, "Bottom", $iBottom)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipBitmapLockBits", "hwnd", $hBitmap, "ptr", $pRect, "uint", $iFlags, "uint", $iFormat, "ptr", $pData)
  Return SetError($aResult[0], 0, $tData)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Unlocks a portion of a bitmap that was locked by _GDIP_BitmapLockBits
; Parameters ....: $hBitmap     - Handle to a bitmap object
;                  $pBitmapData - tagGDIPBITMAPDATA structure previously passed to _GDIP_BitmapLockBits
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the locked portion, call _GDIP_BitmapUnlockBits to release the locked region
; Related .......: _API_DeleteObject
; ===============================================================================================================================
Func _GDIP_BitmapUnlockBits($hBitmap, $tBitmapData)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipBitmapUnlockBits", "hwnd", $hBitmap, "int_ptr", DllStructGetPtr($tBitmapData))
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Clone a Brush object
; Parameters ....: $hBrush      - Handle to a Brush object
; Return values .: Success      - Handle to a new Brush object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Brush object, call _GDIP_BrushDispose to release the resources
; Related .......: _GDIP_BrushDispose
; ===============================================================================================================================
Func _GDIP_BrushClone($hBrush)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCloneBrush", "hwnd", $hBrush, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a solid Brush object
; Parameters ....: $iARGB       - Alpha, Red, Green and Blue components of brush
; Return values .: Success      - Handle to a solid Brush object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Brush object, call _GDIP_BrushDispose to release the resources
; Related .......: _GDIP_BrushDispose
; ===============================================================================================================================
Func _GDIP_BrushCreateSolid($iARGB=0xFF000000)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateSolidFill", "int", $iARGB, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a Brush object
; Parameters ....: $hBrush      - Handle to a Brush object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_BrushCreateSolid
; ===============================================================================================================================
Func _GDIP_BrushDispose($hBrush)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteBrush", "hwnd", $hBrush)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Retrieve the type of Brush object
; Parameters ....: $hBrush      - Handle to a Brush object
; Return values .: Success      - Brush type:
;                  |0 - Solid color
;                  |1 - Hatch fill
;                  |2 - Texture fill
;                  |3 - Path gradient
;                  |4 - Linear gradient
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_BrushGetType($hBrush)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetBrushType", "hwnd", $hBrush, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a custom line cap object
; Parameters ....: $hCap        - Handle to a custom line cap object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_CustomLineCapDispose($hCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteCustomLineCap", "hwnd", $hCap)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get an array of information about the available image decoders
; Parameters ....:
; Return values .: Success      - Array with the following format:
;                  |[0][ 0] - Number of decoders
;                  |[1][ 1] - Codec identifier
;                  |[1][ 2] - File format identifier
;                  |[1][ 3] - Codec name
;                  |[1][ 4] - Dll in which the code resides
;                  |[1][ 5] - The name of the file format used by the codec
;                  |[1][ 6] - Filename extensions associated with the codec
;                  |[1][ 7] - The mime type of the codec
;                  |[1][ 8] - Combination of $GDIP_ICF flags
;                  |[1][ 9] - The version of the codec
;                  |[1][10] - The number of signatures used by the file format
;                  |[1][11] - The number of bytes in each signature
;                  |[1][12] - Pointer to the pattern for each signature
;                  |[1][13] - Pointer to the mask for each signature
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_Encoders
; ===============================================================================================================================
Func _GDIP_Decoders()
  Local $iI, $iCount, $iSize, $pBuffer, $tBuffer, $tCodec, $aResult, $aInfo[1][14]

  $iCount  = _GDIP_DecodersGetCount()
  $iSize   = _GDIP_DecodersGetSize ()
  $tBuffer = DllStructCreate("byte[" & $iSize & "]")
  $pBuffer = DllStructGetPtr($tBuffer)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageDecoders", "int", $iCount, "int", $iSize, "ptr", $pBuffer)
  if $aResult[0] <> 0 then Return SetError($aResult[0], 0, $aInfo)

  Dim $aInfo[$iCount + 1][14]
  $aInfo[0][0] = $iCount
  for $iI = 1 to $iCount
    $tCodec = DllStructCreate($tagGDIPIMAGECODECINFO, $pBuffer)
    $aInfo[$iI][ 1] = _API_StringFromGUID(DllStructGetPtr($tCodec, "CLSID"   ))
    $aInfo[$iI][ 2] = _API_StringFromGUID(DllStructGetPtr($tCodec, "FormatID"))
    $aInfo[$iI][ 3] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "CodecName" ))
    $aInfo[$iI][ 4] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "DllName"   ))
    $aInfo[$iI][ 5] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "FormatDesc"))
    $aInfo[$iI][ 6] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "FileExt"   ))
    $aInfo[$iI][ 7] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "MimeType"  ))
    $aInfo[$iI][ 8] = DllStructGetData($tCodec, "Flags"     )
    $aInfo[$iI][ 9] = DllStructGetData($tCodec, "Version"   )
    $aInfo[$iI][10] = DllStructGetData($tCodec, "SigCount"  )
    $aInfo[$iI][11] = DllStructGetData($tCodec, "SigSize"   )
    $aInfo[$iI][12] = DllStructGetData($tCodec, "SigPattern")
    $aInfo[$iI][13] = DllStructGetData($tCodec, "SigMask"   )
    $pBuffer += DllStructGetSize($tCodec)
  next
  Return $aInfo
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the number of available image decoders
; Parameters ....:
; Return values .: Success      - Number of image decoders
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_DecodersGetSize
; ===============================================================================================================================
Func _GDIP_DecodersGetCount()
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageDecodersSize", "int_ptr", 0, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[1])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the total size of the structure that is returned by _GDIP_GetImageDecoders
; Parameters ....:
; Return values .: Success      - Total size, in bytes
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_DecodersGetCount
; ===============================================================================================================================
Func _GDIP_DecodersGetSize()
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageDecodersSize", "int_ptr", 0, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get an array of information about the available image encoders
; Parameters ....:
; Return values .: Success      - Array with the following format:
;                  |[0][ 0] - Number of decoders
;                  |[1][ 1] - Codec identifier
;                  |[1][ 2] - File format identifier
;                  |[1][ 3] - Codec name
;                  |[1][ 4] - Dll in which the code resides
;                  |[1][ 5] - The name of the file format used by the codec
;                  |[1][ 6] - Filename extensions associated with the codec
;                  |[1][ 7] - The mime type of the codec
;                  |[1][ 8] - Combination of $GDIP_ICF flags
;                  |[1][ 9] - The version of the codec
;                  |[1][10] - The number of signatures used by the file format
;                  |[1][11] - The number of bytes in each signature
;                  |[1][12] - Pointer to the pattern for each signature
;                  |[1][13] - Pointer to the mask for each signature
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_Decoders
; ===============================================================================================================================
Func _GDIP_Encoders()
  Local $iI, $iCount, $iSize, $pBuffer, $tBuffer, $tCodec, $aResult, $aInfo[1][14]

  $iCount  = _GDIP_EncodersGetCount()
  $iSize   = _GDIP_EncodersGetSize ()
  $tBuffer = DllStructCreate("byte[" & $iSize & "]")
  $pBuffer = DllStructGetPtr($tBuffer)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageEncoders", "int", $iCount, "int", $iSize, "ptr", $pBuffer)
  if $aResult[0] <> 0 then Return SetError($aResult[0], 0, $aInfo)

  Dim $aInfo[$iCount + 1][14]
  $aInfo[0][0] = $iCount
  for $iI = 1 to $iCount
    $tCodec = DllStructCreate($tagGDIPIMAGECODECINFO, $pBuffer)
    $aInfo[$iI][ 1] = _API_StringFromGUID(DllStructGetPtr($tCodec, "CLSID"   ))
    $aInfo[$iI][ 2] = _API_StringFromGUID(DllStructGetPtr($tCodec, "FormatID"))
    $aInfo[$iI][ 3] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "CodecName" ))
    $aInfo[$iI][ 4] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "DllName"   ))
    $aInfo[$iI][ 5] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "FormatDesc"))
    $aInfo[$iI][ 6] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "FileExt"   ))
    $aInfo[$iI][ 7] = _API_WideCharToMultiByte(DllStructGetData($tCodec, "MimeType"  ))
    $aInfo[$iI][ 8] = DllStructGetData($tCodec, "Flags"     )
    $aInfo[$iI][ 9] = DllStructGetData($tCodec, "Version"   )
    $aInfo[$iI][10] = DllStructGetData($tCodec, "SigCount"  )
    $aInfo[$iI][11] = DllStructGetData($tCodec, "SigSize"   )
    $aInfo[$iI][12] = DllStructGetData($tCodec, "SigPattern")
    $aInfo[$iI][13] = DllStructGetData($tCodec, "SigMask"   )
    $pBuffer += DllStructGetSize($tCodec)
  next
  Return $aInfo
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Return the encoder CLSID for a specific image file type
; Parameters ....: $sFileExt    - File extension to search for (BMP, JPG, TIF, etc.)
; Return values .: Success      - CLSID of the encoder in string format
;                  Failure      - Blank string
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_EncodersGetCLSID($sFileExt)
  Local $iI, $aEncoders

  $aEncoders = _GDIP_Encoders()
  for $iI = 1 to $aEncoders[0][0]
    if StringInStr($aEncoders[$iI][6], "*." & $sFileExt) > 0 then Return $aEncoders[$iI][1]
  next
  Return SetError(-1, -1, "")
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the number of available image encoders
; Parameters ....:
; Return values .: Success      - Number of image encoders
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_EncodersGetSize
; ===============================================================================================================================
Func _GDIP_EncodersGetCount()
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageEncodersSize", "int_ptr", 0, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[1])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the parameter list for a specified image encoder
; Parameters ....: $hImage      - Handle to the image
;                  $sEncoder    - GUID string of encoder to be used
; Return values .: Success      - tagGDIPPENCODERPARAMS structure
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_EncodersGetParamListSize, tagGDIENCODEERPARAMS
; ===============================================================================================================================
Func _GDIP_EncodersGetParamList($hImage, $sEncoder)
  Local $iSize, $pBuffer, $tBuffer, $pGUID, $tGUID, $aResult

  $iSize   = _GDIP_EncodersGetParamListSize($hImage, $sEncoder)
  if @Error then Return SetError(-1, @Error, 0)
  $tGUID   = _API_GUIDFromString($sEncoder)
  $pGUID   = DllStructGetPtr($tGUID)
  $tBuffer = DllStructCreate("dword Count;byte Params[" & $iSize - 4 & "]")
  $pBuffer = DllStructGetPtr($tBuffer)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetEncoderParameterList", "hwnd", $hImage, "ptr", $pGUID, "int", $iSize, "ptr", $pBuffer)
  Return SetError($aResult[0], 0, $tBuffer)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the size of the parameter list for a specified image encoder
; Parameters ....: $hImage      - Handle to the image
;                  $sEncoder    - GUID string of encoder to be used
; Return values .: Success      - Size, in bytes, of the parameter list
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_EncodersGetParamList
; ===============================================================================================================================
Func _GDIP_EncodersGetParamListSize($hImage, $sEncoder)
  Local $pGUID, $tGUID, $aResult

  $tGUID   = _API_GUIDFromString($sEncoder)
  $pGUID   = DllStructGetPtr($tGUID)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetEncoderParameterListSize", "hwnd", $hImage, "ptr", $pGUID, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[3])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the total size of the structure that is returned by _GDIP_GetImageEncoders
; Parameters ....:
; Return values .: Success      - Total size, in bytes
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_EncodersGetCount
; ===============================================================================================================================
Func _GDIP_EncodersGetSize()
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageEncodersSize", "int_ptr", 0, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a Font object
; Parameters ....: $hFamily     - Handle to a Font Family object
;                  $nSize       - The size of the font measured in the units specified in the $iUnit parameter
;                  $iStyle      - The style of the typeface. Can be a combination of the following:
;                  |0 - Normal weight or thickness of the typeface
;                  |1 - Bold typeface
;                  |2 - Italic typeface
;                  |4 - Underline
;                  |8 - Strikethrough
;                  $iUnit       - Unit of measurement for the font size:
;                  |0 - World coordinates, a nonphysical unit
;                  |1 - Display units
;                  |2 - A unit is 1 pixel
;                  |3 - A unit is 1 point or 1/72 inch
;                  |4 - A unit is 1 inch
;                  |5 - A unit is 1/300 inch
;                  |6 - A unit is 1 millimeter
; Return values .: Success      - Handle to a Font object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Font object, call _GDIP_FontDispose to release the resources
; Related .......: _GDIP_FontDispose
; ===============================================================================================================================
Func _GDIP_FontCreate($hFamily, $nSize, $iStyle=0, $iUnit=3)
  Local $iSize, $aResult

  $iSize   = _Lib_FloatToInt($nSize)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateFont", "hwnd", $hFamily, "int", $iSize, "int", $iStyle, "int", $iUnit, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[5])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a Font object
; Parameters ....: $hFont       - Handle to a Font object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_FontCreate
; ===============================================================================================================================
Func _GDIP_FontDispose($hFont)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteFont", "hwnd", $hFont)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a Font Family object
; Parameters ....: $sFamily     - Name of the Font Family
; Return values .: Success      - Handle to a Font Family object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Font Family object, call _GDIP_FontFamilyDispose to release the resources
; Related .......: _GDIP_FontFamilyDispose
; ===============================================================================================================================
Func _GDIP_FontFamilyCreate($sFamily)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateFontFamilyFromName", "wstr", $sFamily, "ptr", 0, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[3])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a Font Family object
; Parameters ....: $hFamily     - Handle to a Font Family object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_FontFamilyCreate
; ===============================================================================================================================
Func _GDIP_FontFamilyDispose($hFamily)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteFontFamily", "hwnd", $hFamily)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Clears a Graphics object to a specified color
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iARGB       - Alpha, Red, Green and Blue components of color
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ColorCreate
; ===============================================================================================================================
Func _GDIP_GraphicsClear($hGraphics, $iARGB=0xFF000000)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGraphicsClear", "hwnd", $hGraphics, "int", $iARGB)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a Graphics object from a device contect (DC)
; Parameters ....: $hDC         - Device context
; Return values .: Success      - Handle to a Graphics object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Graphics object, call _GDIP_GraphicsDispose to release the resources
; Related .......: _GDIP_GraphicsDispose
; ===============================================================================================================================
Func _GDIP_GraphicsCreateFromHDC($hDC)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateFromHDC", "hwnd", $hDC, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a Graphics object from a window handle
; Parameters ....: $hWnd        - Handle to a window
; Return values .: Success      - Handle to a Graphics object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the Graphics object, call _GDIP_GraphicsDispose to release the resources
; Related .......: _GDIP_GraphicsDispose
; ===============================================================================================================================
Func _GDIP_GraphicsCreateFromHWND($hWnd)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateFromHWND", "hwnd", $hWnd, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a Graphics object
; Parameters ....: $hGraphics   - Handle to a Graphics object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_GrahpicsCreate
; ===============================================================================================================================
Func _GDIP_GraphicsDispose($hGraphics)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteGraphics", "hwnd", $hGraphics)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw an arc
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX          - The X coordinate of the upper left corner of the rectangle that bounds the ellipse in which  to
;                  +draw the arc
;                  $iY          - The Y coordinate of the upper left corner of the rectangle that bounds the ellipse in which  to
;                  +draw the arc
;                  $iWidth      - The width of the rectangle that bounds the ellipse in which to draw the arc
;                  $iHeight     - The height of the rectangle that bounds the ellipse in which to draw the arc
;                  $nStartAngle - The angle between the X axis and the starting point of the arc
;                  $nSweepAngle - The angle between the starting and ending points of the arc
;                  $hPen        - Handle to a pen object that is used to draw the arc.  If 0, a solid black pen with a width of 1
;                  +will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawArc($hGraphics, $iX, $iY, $iWidth, $iHeight, $nStartAngle, $nSweepAngle, $hPen=0)
  Local $iStart, $iSweep, $aResult

  _GDIP_PenDefCreate($hPen)
  $iStart  = _Lib_FloatToInt($nStartAngle)
  $iSweep  = _Lib_FloatToInt($nSweepAngle)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawArcI", "hwnd", $hGraphics, "hwnd", $hPen, "int", $iX, "int", $iY, _
                     "int", $iWidth, "int", $iHeight, "int", $iStart, "int", $iSweep)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a bezier spline
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $hPen        - Handle to a pen object that is used to draw the arc
;                  $iX1         - X coordinate of the starting point
;                  $iY1         - Y coordinate of the starting point
;                  $iX2         - X coordinate of the first control point
;                  $iY2         - Y coordinate of the first control point
;                  $iX3         - X coordinate of the second control point
;                  $iY3         - Y coordinate of the second control point
;                  $iX4         - X coordinate of the ending point
;                  $iY4         - Y coordinate of the ending point
;                  $hPen        - Handle to a pen object that is used to draw the bezier. If 0, a solid black pen with a width of
;                  +1 will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: A Bezier spline does not pass through its control points. The control points act as magnets, pulling the curve
;                  in certain directions to influence the way the spline bends.
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawBezier($hGraphics, $iX1, $iY1, $iX2, $iY2, $iX3, $iY3, $iX4, $iY4, $hPen=0)
  Local $aResult

  _GDIP_PenDefCreate($hPen)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawBezierI", "hwnd", $hGraphics, "hwnd", $hPen, "int", $iX1, "int", $iY1, _
                     "int", $iX2, "int", $iY2, "int", $iX3, "int", $iY3, "int", $iX4, "int", $iY4)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a closed cardinal spline
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $aPoints     - Array that specifies the points of the curve:
;                  |[0][0] - Number of points
;                  |[1][0] - Point 1 X position
;                  |[1][1] - Point 1 Y position
;                  |[2][0] - Point 2 X position
;                  |[2][1] - Point 2 Y position
;                  |[n][0] - Point n X position
;                  |[n][1] - Point n Y position
;                  $hPen        - Handle to a pen object that is used to draw the spline. If 0, a solid black pen with a width of
;                  +1 will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: In a closed cardinal spline, the curve continues through the last point in the points array and connects  with
;                  the first point in the array. The array of points must contain a minimum of three elements.
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawClosedCurve($hGraphics, $aPoints, $hPen=0)
  Local $iI, $iCount, $pPoints, $tPoints, $aResult

  $iCount  = $aPoints[0][0]
  $tPoints = DllStructCreate("int[" & $iCount * 2 & "]")
  $pPoints = DllStructGetPtr($tPoints)
  for $iI = 1 to $iCount
    DllStructSetData($tPoints, 1, $aPoints[$iI][0], (($iI - 1) * 2) + 1)
    DllStructSetData($tPoints, 1, $aPoints[$iI][1], (($iI - 1) * 2) + 2)
  next

  _GDIP_PenDefCreate($hPen)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawClosedCurveI", "hwnd", $hGraphics, "hwnd", $hPen, "ptr", $pPoints, "int", $iCount)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a cardinal spline
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $aPoints     - Array that specifies the points of the curve:
;                  |[0][0] - Number of points
;                  |[1][0] - Point 1 X position
;                  |[1][1] - Point 1 Y position
;                  |[2][0] - Point 2 X position
;                  |[2][1] - Point 2 Y position
;                  |[n][0] - Point n X position
;                  |[n][1] - Point n Y position
;                  $hPen        - Handle to a pen object that is used to draw the spline. If 0, a solid black pen with a width of
;                  +1 will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: A segment is defined as a curve that connects two consecutive points in the cardinal spline.  The ending point
;                  of each segment is the starting point for the next.
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawCurve($hGraphics, $aPoints, $hPen=0)
  Local $iI, $iCount, $pPoints, $tPoints, $aResult

  $iCount  = $aPoints[0][0]
  $tPoints = DllStructCreate("int[" & $iCount * 2 & "]")
  $pPoints = DllStructGetPtr($tPoints)
  for $iI = 1 to $iCount
    DllStructSetData($tPoints, 1, $aPoints[$iI][0], (($iI - 1) * 2) + 1)
    DllStructSetData($tPoints, 1, $aPoints[$iI][1], (($iI - 1) * 2) + 2)
  next

  _GDIP_PenDefCreate($hPen)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawCurveI", "hwnd", $hGraphics, "hwnd", $hPen, "ptr", $pPoints, "int", $iCount)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw an ellipse
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX          - The X coordinate of the upper left corner of the rectangle that bounds the ellipse
;                  $iY          - The Y coordinate of the upper left corner of the rectangle that bounds the ellipse
;                  $iWidth      - The width of the rectangle that bounds the ellipse
;                  $iHeight     - The height of the rectangle that bounds the ellipse
;                  $hPen        - Handle to a pen object that is used to draw the arc.  If 0, a solid black pen with a width of 1
;                  +will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawEllipse($hGraphics, $iX, $iY, $iWidth, $iHeight, $hPen=0)
  Local $aResult

  _GDIP_PenDefCreate($hPen)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawEllipseI", "hwnd", $hGraphics, "hwnd", $hPen, "int", $iX, "int", $iY, _
                     "int", $iWidth, "int", $iHeight)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw an Image object
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $hImage      - Handle to an Image object
;                  $iX          - The X coordinate of the upper left corner of the rendered image
;                  $iY          - The Y coordinate of the upper left corner of the rendered image
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawImage($hGraphics, $hImage, $iX, $iY)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawImageI", "hwnd", $hGraphics, "hwnd", $hImage, "int", $iX, "int", $iY)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw an Image object
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $hImage      - Handle to an Image object
;                  $iSrcX       - The X coordinate of the upper left corner of the source image
;                  $iSrcY       - The Y coordinate of the upper left corner of the source image
;                  $iSrcWidth   - Width of the source image
;                  $iSrcHeight  - Height of the source image
;                  $iDstX       - The X coordinate of the upper left corner of the destination image
;                  $iDstY       - The Y coordinate of the upper left corner of the destination image
;                  $iDstWidth   - Width of the destination image
;                  $iDstHeight  - Height of the destination image
;                  $iUnit       - Specifies the unit of measure for the image
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawImageRectRect($hGraphics, $hImage, $iSrcX, $iSrcY, $iSrcWidth, $iSrcHeight, $iDstX, $iDstY, $iDstWidth, $iDstHeight, $iUnit=2)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawImageRectRectI", "hwnd", $hGraphics, "hwnd", $hImage, "int", $iDstX, "int", _
                     $iDstY, "int", $iDstWidth, "int", $iDstHeight, "int", $iSrcX, "int", $iSrcY, "int", $iSrcWidth, "int", _
                     $iSrcHeight, "int", $iUnit, "int", 0, "int", 0, "int", 0)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a line
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX1         - The X coordinate of the starting point of the line
;                  $iY1         - The Y coordinate of the starting point of the line
;                  $iX2         - The X coordinate of the ending point of the line
;                  $iY2         - The Y coordinate of the ending point of the line
;                  $hPen        - Handle to a pen object that is used to draw the arc.  If 0, a solid black pen with a width of 1
;                  +will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawLine($hGraphics, $iX1, $iY1, $iX2, $iY2, $hPen=0)
  Local $aResult

  _GDIP_PenDefCreate($hPen)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawLineI", "hwnd", $hGraphics, "hwnd", $hPen, "int", $iX1, "int", $iY1, "int", $iX2, "int", $iY2)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a pie
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX          - The X coordinate of the upper left corner of the rectangle that bounds the ellipse in which  to
;                  +draw the pie
;                  $iY          - The Y coordinate of the upper left corner of the rectangle that bounds the ellipse in which  to
;                  +draw the pie
;                  $iWidth      - The width of the rectangle that bounds the ellipse in which to draw the pie
;                  $iHeight     - The height of the rectangle that bounds the ellipse in which to draw the pie
;                  $nStartAngle - The angle, in degrees, between the X axis and the starting point of the arc  that  defines  the
;                  +pie. A positive value specifies clockwise rotation.
;                  $nSweepAngle - The angle, in degrees, between the starting and ending points of the arc that defines the  pie.
;                  +A positive value specifies clockwise rotation.
;                  $hPen        - Handle to a pen object that is used to draw the arc.  If 0, a solid black pen with a width of 1
;                  +will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawPie($hGraphics, $iX, $iY, $iWidth, $iHeight, $nStartAngle, $nSweepAngle, $hPen=0)
  Local $iStart, $iSweep, $aResult

  _GDIP_PenDefCreate($hPen)
  $iStart  = _Lib_FloatToInt($nStartAngle)
  $iSweep  = _Lib_FloatToInt($nSweepAngle)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawPieI", "hwnd", $hGraphics, "hwnd", $hPen, "int", $iX, "int", $iY, _
                     "int", $iWidth, "int", $iHeight, "int", $iStart, "int", $iSweep)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a polygon
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $aPoints     - Array that specify the vertices of the polygon:
;                  |[0][0] - Number of vertices
;                  |[1][0] - Vertice 1 X position
;                  |[1][1] - Vertice 1 Y position
;                  |[2][0] - Vertice 2 X position
;                  |[2][1] - Vertice 2 Y position
;                  |[n][0] - Vertice n X position
;                  |[n][1] - Vertice n Y position
;                  $hPen        - Handle to a pen object that is used to draw the polygon.  If 0, a solid black pen with a  width
;                  +of 1 will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: If the first and last points are not identical, a line is drawn between them to close the polygon
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawPolygon($hGraphics, $aPoints, $hPen=0)
  Local $iI, $iCount, $pPoints, $tPoints, $aResult

  $iCount  = $aPoints[0][0]
  $tPoints = DllStructCreate("int[" & $iCount * 2 & "]")
  $pPoints = DllStructGetPtr($tPoints)
  for $iI = 1 to $iCount
    DllStructSetData($tPoints, 1, $aPoints[$iI][0], (($iI - 1) * 2) + 1)
    DllStructSetData($tPoints, 1, $aPoints[$iI][1], (($iI - 1) * 2) + 2)
  next

  _GDIP_PenDefCreate($hPen)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawPolygonI", "hwnd", $hGraphics, "hwnd", $hPen, "ptr", $pPoints, "int", $iCount)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a rectangle
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX          - The X coordinate of the upper left corner of the rectangle
;                  $iY          - The Y coordinate of the upper left corner of the rectangle
;                  $iWidth      - The width of the rectangle
;                  $iHeight     - The height of the rectangle
;                  $hPen        - Handle to a pen object that is used to draw the rectangle. If 0, a solid black pen with a width
;                  +of 1 will be used
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawRect($hGraphics, $iX, $iY, $iWidth, $iHeight, $hPen=0)
  Local $aResult

  _GDIP_PenDefCreate($hPen)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawRectangleI", "hwnd", $hGraphics, "hwnd", $hPen, "int", $iX, "int", $iY, _
                     "int", $iWidth, "int", $iHeight)
  _GDIP_PenDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a string
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $sString     - String to be drawn
;                  $nX          - X coordinate where the string will be drawn
;                  $nY          - Y coordinate where the string will be drawn
;                  $sFont       - Name of the font to use for drawing
;                  $nSize       - Font size to use for drawing
;                  $iFormat     - Format flags. Can be one or more of the following:
;                  |0x0001 - Specifies that reading order is right to left
;                  |0x0002 - Specifies that individual lines of text are drawn vertically on the display device
;                  |0x0004 - Specifies that parts of characters are allowed to overhang the string's layout rectangle
;                  |0x0020 - Specifies that Unicode layout control characters are displayed with a representative character
;                  |0x0400 - Specifies that an alternate font is used for characters that are not supported in the requested font
;                  |0x0800 - Specifies that the space at the end of each line is included in a string measurement
;                  |0x1000 - Specifies that the wrapping of text to the next line is disabled
;                  |0x2000 - Specifies that only entire lines are laid out in the layout rectangle
;                  |0x4000 - Specifies that characters overhanging the layout rectangle and text  extending  outside  the  layout
;                  +rectangle are allowed to show
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsDrawString($hGraphics, $sString, $nX, $nY, $sFont="Arial", $nSize=10, $iFormat=0)
  Local $hBrush, $iError, $hFamily, $hFont, $hFormat, $aInfo, $tLayout, $bResult

  $hBrush  = _GDIP_BrushCreateSolid()
  $hFormat = _GDIP_StringFormatCreate($iFormat)
  $hFamily = _GDIP_FontFamilyCreate($sFont)
  $hFont   = _GDIP_FontCreate($hFamily, $nSize)
  $tLayout = _GDIP_RectFCreate($nX, $nY, 0, 0)
  $aInfo   = _GDIP_GraphicsMeasureString($hGraphics, $sString, $hFont, $tLayout , $hFormat)
  $bresult = _GDIP_GraphicsDrawStringEx ($hGraphics, $sString, $hFont, $aInfo[0], $hFormat, $hBrush)
  $iError  = @Error
  _GDIP_FontDispose($hFont)
  _GDIP_FontFamilyDispose($hFamily)
  _GDIP_StringFormatDispose($hFormat)
  _GDIP_BrushDispose($hBrush)
  Return SetError($iError, 0, $bResult)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Draw a string
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $sString     - String to be drawn
;                  $hFont       - Handle to the font to use to draw the string
;                  $tLayout     - tagGDIPRECTF structure that bounds the string
;                  $hFormat     - Handle to the string format to draw the string
;                  $hBrush      - Handle to the brush to draw the string
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: tagGDIPRECTF
; ===============================================================================================================================
Func _GDIP_GraphicsDrawStringEx($hGraphics, $sString, $hFont, $tLayout, $hFormat, $hBrush)
  Local $pLayout, $aResult

  $pLayout = DllStructGetPtr($tLayout)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDrawString", "hwnd", $hGraphics, "wstr", $sString, "int", -1, "hwnd", $hFont, _
                     "ptr", $pLayout, "hwnd", $hFormat, "hwnd", $hBrush)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Fill a closed cardinal spline
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $aPoints     - Array that specifies the points of the curve:
;                  |[0][0] - Number of points
;                  |[1][0] - Point 1 X position
;                  |[1][1] - Point 1 Y position
;                  |[2][0] - Point 2 X position
;                  |[2][1] - Point 2 Y position
;                  |[n][0] - Point n X position
;                  |[n][1] - Point n Y position
;                  $hBrush      - Handle to a brush object that is used to fill the ellipse. If 0, a black brush will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: In a closed cardinal spline, the curve continues through the last point in the points array and connects  with
;                  the first point in the array. The array of points must contain a minimum of three elements.
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsFillClosedCurve($hGraphics, $aPoints, $hBrush=0)
  Local $iI, $iCount, $pPoints, $tPoints, $aResult

  $iCount  = $aPoints[0][0]
  $tPoints = DllStructCreate("int[" & $iCount * 2 & "]")
  $pPoints = DllStructGetPtr($tPoints)
  for $iI = 1 to $iCount
    DllStructSetData($tPoints, 1, $aPoints[$iI][0], (($iI - 1) * 2) + 1)
    DllStructSetData($tPoints, 1, $aPoints[$iI][1], (($iI - 1) * 2) + 2)
  next

  _GDIP_BrushDefCreate($hBrush)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipFillClosedCurveI", "hwnd", $hGraphics, "hwnd", $hBrush, "ptr", $pPoints, "int", $iCount)
  _GDIP_BrushDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Fill an ellipse
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX          - The X coordinate of the upper left corner of the rectangle that bounds the ellipse
;                  $iY          - The Y coordinate of the upper left corner of the rectangle that bounds the ellipse
;                  $iWidth      - The width of the rectangle that bounds the ellipse
;                  $iHeight     - The height of the rectangle that bounds the ellipse
;                  $hBrush      - Handle to a brush object that is used to fill the ellipse. If 0, a black brush will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsFillEllipse($hGraphics, $iX, $iY, $iWidth, $iHeight, $hBrush=0)
  Local $aResult

  _GDIP_BrushDefCreate($hBrush)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipFillEllipseI", "hwnd", $hGraphics, "hwnd", $hBrush, "int", $iX, "int", $iY, _
                     "int", $iWidth, "int", $iHeight)
  _GDIP_BrushDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Fill a pie
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX          - The X coordinate of the upper left corner of the rectangle that bounds the ellipse in which  to
;                  +draw the pie
;                  $iY          - The Y coordinate of the upper left corner of the rectangle that bounds the ellipse in which  to
;                  +draw the pie
;                  $iWidth      - The width of the rectangle that bounds the ellipse in which to draw the pie
;                  $iHeight     - The height of the rectangle that bounds the ellipse in which to draw the pie
;                  $nStartAngle - The angle, in degrees, between the X axis and the starting point of the arc  that  defines  the
;                  +pie. A positive value specifies clockwise rotation.
;                  $nSweepAngle - The angle, in degrees, between the starting and ending points of the arc that defines the  pie.
;                  +A positive value specifies clockwise rotation.
;                  $hBrush      - Handle to a brush object that is used to fill the pie. If 0, a black brush will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsFillPie($hGraphics, $iX, $iY, $iWidth, $iHeight, $nStartAngle, $nSweepAngle, $hBrush=0)
  Local $iStart, $iSweep, $aResult

  _GDIP_BrushDefCreate($hBrush)
  $iStart  = _Lib_FloatToInt($nStartAngle)
  $iSweep  = _Lib_FloatToInt($nSweepAngle)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipFillPieI", "hwnd", $hGraphics, "hwnd", $hBrush, "int", $iX, "int", $iY, _
                     "int", $iWidth, "int", $iHeight, "int", $iStart, "int", $iSweep)
  _GDIP_BrushDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Fill a rectangle
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iX          - The X coordinate of the upper left corner of the rectangle
;                  $iY          - The Y coordinate of the upper left corner of the rectangle
;                  $iWidth      - The width of the rectangle
;                  $iHeight     - The height of the rectangle
;                  $hBrush      - Handle to a brush object that is used to fill the rectangle. If 0, a black brush will be used.
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsFillRect($hGraphics, $iX, $iY, $iWidth, $iHeight, $hBrush=0)
  Local $aResult

  _GDIP_BrushDefCreate($hBrush)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipFillRectangleI", "hwnd", $hGraphics, "hwnd", $hBrush, "int", $iX, "int", $iY, _
                     "int", $iWidth, "int", $iHeight)
  _GDIP_BrushDefDispose()
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets a handle to the device context of the Graphics object
; Parameters ....: $hGraphics   - Handle to a Graphics object
; Return values .: Success      - DC of the Graphics object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: Each call to the _GDIP_GraphicsGetDC should be paired with a call to the _GDIP_GraphicsReleaseDC.  Do not call
;                  any methods of the Graphics object between the calls.
; Related .......: _GDIP_GraphicsReleaseDC
; ===============================================================================================================================
Func _GDIP_GraphicsGetDC($hGraphics)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetDC", "hwnd", $hGraphics, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the graphics object rendering quality
; Parameters ....: $hGraphics   - Handle to a Graphics object
; Return values .: Success      - Smoothing mode. Can be one of the following:
;                  |0 - Smoothing is not applied
;                  |1 - Smoothing is applied using an 8 X 4 box filter
;                  |2 - Smoothing is applied using an 8 X 8 box filter
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_GraphicsSetSmoothingMode
; ===============================================================================================================================
Func _GDIP_GraphicsGetSmoothingMode($hGraphics)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetSmoothingMode", "hwnd", $hGraphics, "int_ptr", 0)
  Switch $aResult[2]
    case 3
      Return SetError($aResult[0], 0, 1)
    case 7
      Return SetError($aResult[0], 0, 2)
    case else
      Return SetError($aResult[0], 0, 0)
  EndSwitch
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Measures the size of a string
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $sString     - String to be drawn
;                  $hFont       - Handle to the font to use to draw the string
;                  $tLayout     - tagGDIPRECTF structure that bounds the string
;                  $hFormat     - Handle to the string format to draw the string
; Return values .: Success      - Array with the following format
;                  |[0] - tagGDIPRECTF structure that receives the rectangle that bounds the string
;                  |[1] - The number of characters that actually fit into the layout rectangle
;                  |[2] - The number of lines that fit into the layout rectangle
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: tagGDIPRECTF
; ===============================================================================================================================
Func _GDIP_GraphicsMeasureString($hGraphics, $sString, $hFont, $tLayout, $hFormat)
  Local $pLayout, $pRectF, $tRectF, $aResult, $aInfo[3]

  $pLayout = DllStructGetPtr($tLayout)
  $tRectF  = DllStructCreate($tagGDIPRECTF)
  $pRectF  = DllStructGetPtr($tRectF )
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipMeasureString", "hwnd", $hGraphics, "wstr", $sString, "int", -1, "hwnd", $hFont, _
                     "ptr", $pLayout, "hwnd", $hFormat, "ptr", $pRectF, "int_ptr", 0, "int_ptr", 0)
  $aInfo[0] = $tRectF
  $aInfo[1] = $aResult[8]
  $aInfo[2] = $aResult[9]
  Return SetError($aResult[0], 0, $aInfo)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Releases the device context of the Graphics object
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $hDC         - Handle to the Graphics device context
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_GraphicsGetDC
; ===============================================================================================================================
Func _GDIP_GraphicsReleaseDC($hGraphics, $hDC)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipReleaseDC", "hwnd", $hGraphics, "hwnd", $hDC)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the world transformation for a graphics object
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $hMatrix     - Handle to a Matrix object that specifies the world transformation
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_GraphicsSetTransform($hGraphics, $hMatrix)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetWorldTransform", "hwnd", $hGraphics, "hwnd", $hMatrix)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the graphics object rendering quality
; Parameters ....: $hGraphics   - Handle to a Graphics object
;                  $iSmooth     - Smoothing mode:
;                  |0 - Smoothing is not applied
;                  |1 - Smoothing is applied using an 8 X 4 box filter
;                  |2 - Smoothing is applied using an 8 X 8 box filter
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: Smoothing performed by an 8 X 4 box filter gives better results for nearly vertical lines  than  it  does  for
;                  nearly horizontal lines.  Smoothing performed by an 8 X 8 box filter gives equally  good  results  for  nearly
;                  vertical and nearly horizontal lines.  The 8x8 algorithm produces higher quality smoothing but is slower  than
;                  the 8 X 4 algorithm.
; Related .......: _GDIP_GraphicsGetSmoothingMode
; ===============================================================================================================================
Func _GDIP_GraphicsSetSmoothingMode($hGraphics, $iSmooth)
  Local $iMode, $aResult

  Switch $iSmooth
    case 1
      $iMode = 3
    case 2
      $iMode = 7
    case else
      $iMode = 0
  EndSwitch
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetSmoothingMode", "hwnd", $hGraphics, "int", $iMode)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release an image object
; Parameters ....: $hImage      - Handle to an image object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_ImageDispose($hImage)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDisposeImage", "hwnd", $hImage)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the graphics context of the image
; Parameters ....: $hImage      - Handle to an image object
; Return values .: Success      - Handle to a Graphics object
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_ImageGetGraphicsContext($hImage)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageGraphicsContext", "hwnd", $hImage, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the image height
; Parameters ....: $hImage      - Handle to an image object
; Return values .: Success      - Image height, in pixels
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ImageGetWidth
; ===============================================================================================================================
Func _GDIP_ImageGetHeight($hImage)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageHeight", "hwnd", $hImage, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Get the image width
; Parameters ....: $hImage      - Handle to am image object
; Return values .: Success      - Image width, in pixels
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ImageGetHeight
; ===============================================================================================================================
Func _GDIP_ImageGetWidth($hImage)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetImageWidth", "hwnd", $hImage, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create an image object based on a file
; Parameters ....: $sFileName   - Fully qualified image file name
; Return values .: Success      - Handle to the new image object
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ImageSaveToFile
; ===============================================================================================================================
Func _GDIP_ImageLoadFromFile($sFileName)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipLoadImageFromFile", "wstr", $sFileName, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Save an image to file
; Parameters ....: $hImage      - Handle to an image object
;                  $sFileName   - Fully qualified image file name
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ImageLoadFromFile
; ===============================================================================================================================
Func _GDIP_ImageSaveToFile($hImage, $sFileName)
  Local $sCLSID, $sExt

  $sExt   = _Str_ExtractFileExt($sFileName)
  $sCLSID = _GDIP_EncodersGetCLSID($sExt)
  if $sCLSID = "" then Return SetError(-1, 0, False)
  _GDIP_ImageSaveToFileEx($hImage, $sFileName, $sCLSID, 0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Save an image to file
; Parameters ....: $hImage      - Handle to an image object
;                  $sFileName   - Fully qualified image file name
;                  $sEncoder    - GUID string of encoder to be used
;                  $pParams     - Pointer to a tagGDIPPENCODERPARAMS structure
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ImageLoadFromFile, tagGDIPPENCODERPARAMS
; ===============================================================================================================================
Func _GDIP_ImageSaveToFileEx($hImage, $sFileName, $sEncoder, $pParams=0)
  Local $pGUID, $tGUID, $aResult

  $tGUID   = _API_GUIDFromString($sEncoder)
  $pGUID   = DllStructGetPtr($tGUID)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSaveImageToFile", "hwnd", $hImage, "wstr", $sFileName, "ptr", $pGUID, "ptr", $pParams)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Creates and initializes a Matrix object that represents the identity matrix
; Parameters ....:
; Return values .: Success      - Handle to a Matrix object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the matrix, call _GDIP_MatrixDispose to release the resources
; Related .......: _GDIP_MatrixDispose
; ===============================================================================================================================
Func _GDIP_MatrixCreate()
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateMatrix", "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[1])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a matrix object
; Parameters ....: $hMatrix     - Handle to a Matrix object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_MatrixCreate
; ===============================================================================================================================
Func _GDIP_MatrixDispose($hMatrix)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteMatrix", "hwnd", $hMatrix)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Updates a matrix with the product of itself and a rotation matrix
; Parameters ....: $hMatrix     - Handle to a Matrix object
;                  $nAngle      - The angle of rotation in degrees. Positive values specify clockwise rotation.
;                  $fAppend     - Specifies the order of the multiplication:
;                  | True - Specifies that the rotation matrix is on the left
;                  |False - Specifies that the rotation matrix is on the right
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_MatrixRotate($hMatrix, $nAngle, $fAppend=False)
  Local $iAngle, $aResult

  $iAngle  = _Lib_FloatToInt($nAngle)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipRotateMatrix", "hwnd", $hMatrix, "int", $iAngle, "int", $fAppend)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Add a value to an enocder parameter list
; Parameters ....: $tParams     - tagGDIPPENCODERPARAMS structure returned from _GDIP_ParamInit
;                  $sGUID       - Encoder parameter GUID. Can be one of the following:
;                  |$GDIP_EPGCHROMINANCETABLE - Chrominance table settings
;                  |$GDIP_EPGCOLORDEPTH       - Color depth settings
;                  |$GDIP_EPGCOMPRESSION      - Compression settings
;                  |$GDIP_EPGLUMINANCETABLE   - Luminance table settings
;                  |$GDIP_EPGQUALITY          - Quality settings
;                  |$GDIP_EPGRENDERMETHOD     - Render method settings
;                  |$GDIP_EPGSAVEFLAG         - Save flag settings
;                  |$GDIP_EPGSCANMETHOD       - Scan mode settings
;                  |$GDIP_EPGTRANSFORMATION   - Transformation settings
;                  |$GDIP_EPGVERSION          - Software version settings
;                  $iCount      - Number of elements in the $pValues array
;                  $iType       - Encoder parameter value type. Can be one of the following:
;                  |$GDIP_EPTBYTE          - 8 bit unsigned integer
;                  |$GDIP_EPTASCII         - Null terminated character string
;                  |$GDIP_EPTSHORT         - 16 bit unsigned integer
;                  |$GDIP_EPTLONG          - 32 bit unsigned integer
;                  |$GDIP_EPTRATIONAL      - Two longs (numerator, denomintor)
;                  |$GDIP_EPTLONGRANGE     - Two longs (low, high)
;                  |$GDIP_EPTUNDEFINED     - Array of bytes of any type
;                  |$GDIP_EPTRATIONALRANGE - Two ratationals (low, high)
;                  $pValues     - Pointer to an array of values. Each value has the type specified by the $iType data member.
; Return values .:
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_ParamInit, tagGDIPPENCODERPARAMS
; ===============================================================================================================================
Func _GDIP_ParamAdd(ByRef $tParams, $sGUID, $iCount, $iType, $pValues)
  Local $tParam

  $tParam = DllStructCreate($tagGDIPENCODERPARAM, DllStructGetPtr($tParams, "Params") + (DllStructGetData($tParams, "Count") * 28))
  _API_GUIDFromStringEx($sGUID, DllStructGetPtr($tParam, "GUID"))
  DllStructSetData($tParam, "Type"  , $iType  )
  DllStructSetData($tParam, "Count" , $iCount )
  DllStructSetData($tParam, "Values", $pValues)
  DllStructSetData($tParams, "Count", DllStructGetData($tParams, "Count") + 1)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Initialize an encoder parameter list
; Parameters ....: $iCount      - The total number of parameters that the list will contain
; Return values .: Success      - tagGDIPPENCODERPARAMS structure
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: In order to pass parameters to any of the encoder functions, you must use  an  encoder  parameter  list.  This
;                  function is used to initialize an encoder parameter list that can then be passed to _GDIP_Param add to add the
;                  actual parameters.
; Related .......: _GDIP_ParamAdd, tagGDIPPENCODERPARAMS
; ===============================================================================================================================
Func _GDIP_ParamInit($iCount)
  if $iCount <= 0 then Return SetError(-1, -1, 0)
  Return DllStructCreate("dword Count;byte Params[" & $iCount * 28 & "]")
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a pen object
; Parameters ....: $iARGB       - Alpha, Red, Green and Blue components of pen color
;                  $nWidth      - The width of the pen measured in the units specified in the $iUnit parameter
;                  $iUnit       - Unit of measurement for the pen size:
;                  |0 - World coordinates, a nonphysical unit
;                  |1 - Display units
;                  |2 - A unit is 1 pixel
;                  |3 - A unit is 1 point or 1/72 inch
;                  |4 - A unit is 1 inch
;                  |5 - A unit is 1/300 inch
;                  |6 - A unit is 1 millimeter
; Return values .: Success      - Handle to a pen object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the pen, call _GDIP_PenDispose to release the resources
; Related .......: _GDIP_PenDispose
; ===============================================================================================================================
Func _GDIP_PenCreate($iARGB=0xFF000000, $nWidth=1, $iUnit=2)
  Local $iWidth, $aResult

  $iWidth  = _Lib_FloatToInt($nWidth)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreatePen1", "int", $iARGB, "int", $iWidth, "int", $iUnit, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[4])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a pen object
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenCreate
; ===============================================================================================================================
Func _GDIP_PenDispose($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeletePen", "hwnd", $hPen)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the pen alignment
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - Alignment type:
;                  |0 - Specifies that the pen is aligned on the center of the line that is drawn
;                  |1 - Specifies, when drawing a polygon, that the pen is aligned on the inside of the edge of the polygon
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetAlignment
; ===============================================================================================================================
Func _GDIP_PenGetAlignment($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetPenMode", "hwnd", $hPen, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the pen color
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - Pen color
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetColor
; ===============================================================================================================================
Func _GDIP_PenGetColor($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetPenColor", "hwnd", $hPen, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the custom end cap for the pen
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - Handle to a CustomLineCap object that specifies the pen custom end cap
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetCustomEndCap
; ===============================================================================================================================
Func _GDIP_PenGetCustomEndCap($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetPenCustomEndCap", "hwnd", $hPen, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the pen dash cap style
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - Dash cap style:
;                  |$GDIP_DASHCAPFLAT     - A square cap that squares off both ends of each dash
;                  |$GDIP_DASHCAPROUND    - A circular cap that rounds off both ends of each dash
;                  |$GDIP_DASHCAPTRIANGLE - A triangular cap that points both ends of each dash
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetDashCap
; ===============================================================================================================================
Func _GDIP_PenGetDashCap($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetPenDashCap197819", "hwnd", $hPen, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the pen dash style
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - Dash style:
;                  |$GDIP_DASHSTYLESOLID      - A solid line
;                  |$GDIP_DASHSTYLEDASH       - A dashed line
;                  |$GDIP_DASHSTYLEDOT        - A dotted line
;                  |$GDIP_DASHSTYLEDASHDOT    - An alternating dash-dot line
;                  |$GDIP_DASHSTYLEDASHDOTDOT - An alternating dash-dot-dot line
;                  |$GDIP_DASHSTYLECUSTOM     - A a user-defined, custom dashed line
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetDashStyle
; ===============================================================================================================================
Func _GDIP_PenGetDashStyle($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetPenDashStyle", "hwnd", $hPen, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the pen end cap
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - End cap type:
;                  |$GDIP_LINECAPFLAT          - Specifies a flat cap
;                  |$GDIP_LINECAPSQUARE        - Specifies a square cap
;                  |$GDIP_LINECAPROUND         - Specifies a circular cap
;                  |$GDIP_LINECAPTRIANGLE      - Specifies a triangular cap
;                  |$GDIP_LINECAPNOANCHOR      - Specifies that the line ends are not anchored
;                  |$GDIP_LINECAPSQUAREANCHOR  - Specifies that the line ends are anchored with a square
;                  |$GDIP_LINECAPROUNDANCHOR   - Specifies that the line ends are anchored with a circle
;                  |$GDIP_LINECAPDIAMONDANCHOR - Specifies that the line ends are anchored with a diamond
;                  |$GDIP_LINECAPARROWANCHOR   - Specifies that the line ends are anchored with arrowheads
;                  |$GDIP_LINECAPCUSTOM        - Specifies that the line ends are made from a CustomLineCap
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetEndCap
; ===============================================================================================================================
Func _GDIP_PenGetEndCap($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetPenEndCap", "hwnd", $hPen, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[2])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Retrieve the width of a pen
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - Width of pen
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetWidth
; ===============================================================================================================================
Func _GDIP_PenGetWidth($hPen)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipGetPenWidth", "hwnd", $hPen, "int_ptr", 0)
  Return SetError($aResult[0], 0, _Lib_IntToFloat($aResult[2]))
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the pen alignment
; Parameters ....: $hPen        - Handle to a pen object
;                  $iAlignment  - Pen alignment. Can be one of the following:
;                  |0 - Specifies that the pen is aligned on the center of the line that is drawn
;                  |1 - Specifies, when drawing a polygon, that the pen is aligned on the inside of the edge of the polygon
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenGetAlignment
; ===============================================================================================================================
Func _GDIP_PenSetAlignment($hPen, $iAlignment=0)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetPenMode", "hwnd", $hPen, "int", $iAlignment)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the pen color
; Parameters ....: $hPen        - Handle to a pen object
;                  $iARGB       - Alpha, Red, Green and Blue components of pen color
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenGetPenColor
; ===============================================================================================================================
Func _GDIP_PenSetColor($hPen, $iARGB)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetPenColor", "hwnd", $hPen, "int", $iARGB)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the pen dash cap style
; Parameters ....: $hPen        - Handle to a pen object
;                  $iDash       - Dash cap style. Can be one of the following:
;                  |$GDIP_DASHCAPFLAT     - A square cap that squares off both ends of each dash
;                  |$GDIP_DASHCAPROUND    - A circular cap that rounds off both ends of each dash
;                  |$GDIP_DASHCAPTRIANGLE - A triangular cap that points both ends of each dash
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenGetDashCap
; ===============================================================================================================================
Func _GDIP_PenSetDashCap($hPen, $iDash=0)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetPenDashCap197819", "hwnd", $hPen, "int", $iDash)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the custom end cap for the pen
; Parameters ....: $hPen        - Handle to a pen object
;                  $hEndCap     - Handle to a CustomLineCap object that specifies the pen custom end cap
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenGetCustomEndCap
; ===============================================================================================================================
Func _GDIP_PenSetCustomEndCap($hPen, $hEndCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetPenCustomEndCap", "hwnd", $hPen, "hwnd", $hEndCap)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the pen dash style
; Parameters ....: $hPen        - Handle to a pen object
;                  $iStyle      - Dash style. Can be one of the following:
;                  |$GDIP_DASHSTYLESOLID      - A solid line
;                  |$GDIP_DASHSTYLEDASH       - A dashed line
;                  |$GDIP_DASHSTYLEDOT        - A dotted line
;                  |$GDIP_DASHSTYLEDASHDOT    - An alternating dash-dot line
;                  |$GDIP_DASHSTYLEDASHDOTDOT - An alternating dash-dot-dot line
;                  |$GDIP_DASHSTYLECUSTOM     - A a user-defined, custom dashed line
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenSetDashStyle
; ===============================================================================================================================
Func _GDIP_PenSetDashStyle($hPen, $iStyle=0)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetPenDashStyle", "hwnd", $hPen, "int", $iStyle)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Gets the pen end cap
; Parameters ....: $hPen        - Handle to a pen object
;                  $iEndCap     - End cap type. Can be one of the following:
;                  |$GDIP_LINECAPFLAT          - Specifies a flat cap
;                  |$GDIP_LINECAPSQUARE        - Specifies a square cap
;                  |$GDIP_LINECAPROUND         - Specifies a circular cap
;                  |$GDIP_LINECAPTRIANGLE      - Specifies a triangular cap
;                  |$GDIP_LINECAPNOANCHOR      - Specifies that the line ends are not anchored
;                  |$GDIP_LINECAPSQUAREANCHOR  - Specifies that the line ends are anchored with a square
;                  |$GDIP_LINECAPROUNDANCHOR   - Specifies that the line ends are anchored with a circle
;                  |$GDIP_LINECAPDIAMONDANCHOR - Specifies that the line ends are anchored with a diamond
;                  |$GDIP_LINECAPARROWANCHOR   - Specifies that the line ends are anchored with arrowheads
;                  |$GDIP_LINECAPCUSTOM        - Specifies that the line ends are made from a CustomLineCap
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenGetEndCap
; ===============================================================================================================================
Func _GDIP_PenSetEndCap($hPen, $iEndCap)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetPenEndCap", "hwnd", $hPen, "int", $iEndCap)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Sets the width of a pen
; Parameters ....: $hPen        - Handle to a pen object
;                  $nWidth      - Width of pen
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_PenGetWidth
; ===============================================================================================================================
Func _GDIP_PenSetWidth($hPen, $nWidth)
  Local $iWidth, $aResult

  $iWidth  = _Lib_FloatToInt($nWidth)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipSetPenWidth", "hwnd", $hPen, "int", $iWidth)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a tagGDIPRECTF structure
; Parameters ....: $nX      - X coordinate of upper left hand corner of rectangle
;                  $nY      - Y coordinate of upper left hand corner of rectangle
;                  $nWidth  - Rectangle width
;                  $nHeight - Rectangle height
; Return values .: Success      - tagGDIPRECTF structure
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: tagGDIPRECTF
; ===============================================================================================================================
Func _GDIP_RectFCreate($nX=0, $nY=0, $nWidth=0, $nHeight=0)
  Local $tRectF

  $tRectF = DllStructCreate($tagGDIPRECTF)
  DllStructSetData($tRectF, "X"     , $nX     )
  DllStructSetData($tRectF, "Y"     , $nY     )
  DllStructSetData($tRectF, "Width" , $nWidth )
  DllStructSetData($tRectF, "Height", $nHeight)
  Return $tRectF
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Clean up resources used by Microsoft Windows GDI+
; Parameters ....:
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: You must dispose of all of your GDI+ objects before you call _GDIP_Shutdown
; Related .......: _GDIP_Startup
; ===============================================================================================================================
Func _GDIP_Shutdown()
  if $ghA3LGDIPDll = 0 then Return SetError(-1, -1, False)

  $giA3LGDIPRef -= 1
  if $giA3LGDIPRef = 0 then
    DllCall($ghA3LGDIPDll, "none", "GdiplusShutdown", "ptr", $giA3LGDIPToken)
    DllClose($ghA3LGDIPDll)
    $ghA3LGDIPDll = 0
  endif
  Return True
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Initialize Microsoft Windows GDI+
; Parameters ....:
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: Call _GDIP_Startup before you create any GDI+ objects.  GDI+ requires a redistributable for applications  that
;                  run on the Microsoft Windows NT 4.0 SP6, Windows 2000, Windows 98, and Windows Me operating systems.
; Related .......: _GDIP_Shutdown
; ===============================================================================================================================
Func _GDIP_Startup()
  Local $pInput, $tInput, $pToken, $tToken, $aResult

  $giA3LGDIPRef += 1
  if $giA3LGDIPRef > 1 then Return True

  $ghA3LGDIPDll = DllOpen("GDIPlus.dll")
  _Lib_Check("_GDIP_Startup (GDIPlus.dll not found)", @Error, -1)

  $tInput = DllStructCreate($tagGDIPSTARTUPINPUT)
  $pInput = DllStructGetPtr($tInput)
  $tToken = DllStructCreate("int Data")
  $pToken = DllStructGetPtr($tToken)
  DllStructSetData($tInput, "Version", 1)
  $aResult = DllCall($ghA3LGDIPDll, "int", "GdiplusStartup", "ptr", $pToken, "ptr", $pInput, "ptr", 0)
  $giA3LGDIPToken = DllStructGetData($tToken, "Data")
  Return $aResult[0] <> 0
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Create a String Format object
; Parameters ....: $iFormat     - Format flags. Can be one or more of the following:
;                  |0x0001 - Specifies that reading order is right to left
;                  |0x0002 - Specifies that individual lines of text are drawn vertically on the display device
;                  |0x0004 - Specifies that parts of characters are allowed to overhang the string's layout rectangle
;                  |0x0020 - Specifies that Unicode layout control characters are displayed with a representative character
;                  |0x0400 - Specifies that an alternate font is used for characters that are not supported in the requested font
;                  |0x0800 - Specifies that the space at the end of each line is included in a string measurement
;                  |0x1000 - Specifies that the wrapping of text to the next line is disabled
;                  |0x2000 - Specifies that only entire lines are laid out in the layout rectangle
;                  |0x4000 - Specifies that characters overhanging the layout rectangle and text  extending  outside  the  layout
;                  +rectangle are allowed to show
;                  $iLandID     - The language to use
; Return values .: Success      - Handle to a string format object
;                  Failure      - 0
; Author ........: Paul Campbell (PaulIA)
; Remarks .......: When you are done with the String Format object, call _GDIP_StringFormatDispose to release the resources
; Related .......: _GDIP_StringFormatDispose
; ===============================================================================================================================
Func _GDIP_StringFormatCreate($iFormat=0, $iLangID=0)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipCreateStringFormat", "int", $iFormat, "short", $iLangID, "int_ptr", 0)
  Return SetError($aResult[0], 0, $aResult[3])
EndFunc

; #FUNCTION# ====================================================================================================================
; Description ...: Release a String Format object
; Parameters ....: $hFormat     - Handle to a String Format object
; Return values .: Success      - True
;                  Failure      - False
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......: _GDIP_StringFormatCreate
; ===============================================================================================================================
Func _GDIP_StringFormatDispose($hFormat)
  Local $aResult

  $aResult = DllCall($ghA3LGDIPDll, "int", "GdipDeleteStringFormat", "hwnd", $hFormat)
  Return SetError($aResult[0], 0, $aResult[0]=0)
EndFunc

; ===============================================================================================================================
; *** *** *** *** *** *** *** *** *** *** *** *** ***     Helper Functions    *** *** *** *** *** *** *** *** *** *** *** *** ***
; ===============================================================================================================================

; ===============================================================================================================================
; Description ...: Create a default Brush object if needed
; Parameters ....: $hBrush      - Handle to a Brush object
; Return values .: Success      - $hBrush or a default Brush object
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_BrushDefCreate(ByRef $hBrush)
  if $hBrush = 0 then
    $ghA3LGDIPBrush = _GDIP_BrushCreateSolid()
    $hBrush = $ghA3LGDIPBrush
  endif
EndFunc

; ===============================================================================================================================
; Description ...: Free default Brush object
; Parameters ....:
; Return values .:
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_BrushDefDispose()
  if $ghA3LGDIPBrush <> 0 then
    _GDIP_BrushDispose($ghA3LGDIPBrush)
    $ghA3LGDIPBrush = 0
  endif
EndFunc

; ===============================================================================================================================
; Description ...: Create a default Pen object if needed
; Parameters ....: $hPen        - Handle to a pen object
; Return values .: Success      - $hPen or a default pen object
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_PenDefCreate(ByRef $hPen)
  if $hPen = 0 then
    $ghA3LGDIPPen = _GDIP_PenCreate()
    $hPen = $ghA3LGDIPPen
  endif
EndFunc

; ===============================================================================================================================
; Description ...: Free default Pen object
; Parameters ....:
; Return values .:
; Author ........: Paul Campbell (PaulIA)
; Remarks .......:
; Related .......:
; ===============================================================================================================================
Func _GDIP_PenDefDispose()
  if $ghA3LGDIPPen <> 0 then
    _GDIP_PenDispose($ghA3LGDIPPen)
    $ghA3LGDIPPen = 0
  endif
EndFunc

Opt("MustDeclareVars", 0)